@@ -30,8 +30,9 @@ class Client:
     """
 
     def __init__(self, domain="public.md.ai", access_token=None):
-        domain_pattern = r"^\w+\.md\.ai(:\d+)?$"
-        if not re.match(domain_pattern, domain):
+        domain_match = re.match(r"^\w+\.md\.ai$", domain)
+        dev_domain_match = re.match(r"^\w+\.mdai.dev(:\d+)?$", domain)
+        if not domain_match and not dev_domain_match:
             raise ValueError(f"domain {domain} is invalid: should be format *.md.ai")
 
         self.domain = domain
@@ -115,7 +116,12 @@ class Client:
         if num_chunks > 1:
             print(f"Importing {len(annotations)} total annotations in {num_chunks} chunks...")
 
+        failed_annotations = []
+
         for i in range(num_chunks):
+            if num_chunks > 1:
+                print(f"Chunk {i+1}...")
+
             start = i * chunk_size
             end = (i + 1) * chunk_size
             annotations_chunk = annotations[start:end]
@@ -132,6 +138,20 @@ class Client:
             manager.create_job()
             manager.wait_until_ready()
 
+            for failed_annotation in manager.failed_annotations:
+                # add start index since returned index is for chunk
+                failed_annotation["index"] += start
+                failed_annotations.append(failed_annotation)
+
+        if num_chunks > 1:
+            num_failed = len(failed_annotations)
+            print(
+                f"Successfully imported {len(annotations) - num_failed} / {len(annotations)}"
+                + f" total annotations into project {project_id}."
+            )
+
+        return failed_annotations
+
     def _create_headers(self):
         headers = {}
         if self.access_token:
@@ -442,6 +462,9 @@ class AnnotationsImportManager:
 
         self.job_id = None
 
+        # list of failed annotation imports
+        self.failed_annotations = []
+
         # ready threading event
         self._ready = threading.Event()
 
@@ -456,7 +479,12 @@ class AnnotationsImportManager:
             "modelHashId": self.model_id,
             "annotations": self.annotations,
         }
+
+        # reset list of failed annotation imports
+        self.failed_annotations = []
+
         r = self.session.post(endpoint, json=params, headers=self.headers)
+
         if r.status_code == 202:
             self.job_id = r.json()["jobId"]
             msg = f"Importing {len(self.annotations)} annotations into project {self.project_id}"
@@ -469,7 +497,7 @@ class AnnotationsImportManager:
             self._check_job_progress()
         else:
             print(r.status_code)
-            if r.status_code == 401:
+            if r.status_code in (400, 401):
                 msg = "Provided IDs are invalid, or you do not have sufficient permissions."
                 print(msg)
             self._on_job_error()
@@ -549,7 +577,20 @@ class AnnotationsImportManager:
         r = self.session.post(endpoint, json=params, headers=self.headers)
         if r.status_code != 200:
             r.raise_for_status()
-        print(f"Successfully imported annotations into project {self.project_id}.")
+
+        try:
+            body = r.json()
+            self.failed_annotations = body["failed"]
+        except (TypeError, KeyError):
+            self._on_job_error()
+            return
+
+        num_failed = len(self.failed_annotations)
+        print(
+            f"Successfully imported {len(self.annotations) - num_failed} / {len(self.annotations)}"
+            + f" annotations into project {self.project_id}"
+            + f", dataset {self.dataset_id}."
+        )
         # fire ready threading.Event
         self._ready.set()
 
@@ -565,6 +606,9 @@ class AnnotationsImportManager:
         r = self.session.post(endpoint, json=params, headers=self.headers)
         if r.status_code != 200:
             r.raise_for_status()
-        print(f"Error importing annotations into project {self.project_id}.")
+        print(
+            f"Error importing annotations into project {self.project_id}"
+            + f", dataset {self.dataset_id}."
+        )
         # fire ready threading.Event
         self._ready.set()
